@@ -24,6 +24,7 @@ from .basic_types import (
     ReducerType,
     Scheduler,
     ViewDecorator,
+    ViewOptions,
     ViewReturnType,
     is_complete_reducer_result,
     is_state_reducer_result,
@@ -54,10 +55,11 @@ __all__ = (
     'ReducerResult',
     'ReducerType',
     'Scheduler',
+    'Store',
     'ViewDecorator',
+    'ViewOptions',
     'ViewReturnType',
+    'combine_reducers',
     'is_complete_reducer_result',
     'is_state_reducer_result',
-    'combine_reducers',
-    'Store',
 )
