@@ -59,6 +59,7 @@ from redux.basic_types import (
 )
 from redux.serialization_mixin import SerializationMixin
 from redux.side_effect_runner import SideEffectRunnerThread
+from redux.with_state import WithState
 
 if TYPE_CHECKING:
     from collections.abc import Awaitable, Callable
@@ -107,8 +108,6 @@ class Store(Generic[State, Action, Event], SerializationMixin):
 
         self._is_running = Lock()
 
-        self.subscribe_event(FinishEvent, self._handle_finish_event)
-
         if self.store_options.auto_init:
             if self.store_options.scheduler:
                 self.store_options.scheduler(
@@ -154,6 +153,8 @@ class Store(Generic[State, Action, Event], SerializationMixin):
         while len(self._events) > 0:
             event = self._events.pop(0)
             if event is not None:
+                if isinstance(event, FinishEvent):
+                    self._handle_finish_event()
                 for event_handler in self._event_handlers[type(event)].copy():
                     self._event_handlers_queue.put_nowait((event_handler, event))
 
@@ -244,15 +245,20 @@ class Store(Generic[State, Action, Event], SerializationMixin):
         keep_ref: bool = True,
     ) -> Callable[[], None]:
         """Subscribe to state changes."""
+
+        def unsubscribe(_: weakref.ref | None = None) -> None:
+            return self._listeners.remove(listener_ref)
+
         if keep_ref:
             listener_ref = listener
         elif inspect.ismethod(listener):
-            listener_ref = weakref.WeakMethod(listener)
+            listener_ref = weakref.WeakMethod(listener, unsubscribe)
         else:
-            listener_ref = weakref.ref(listener)
+            listener_ref = weakref.ref(listener, unsubscribe)
 
         self._listeners.add(listener_ref)
-        return lambda: self._listeners.remove(listener_ref)
+
+        return unsubscribe
 
     def subscribe_event(
         self: Store[State, Action, Event],
@@ -291,7 +297,7 @@ class Store(Generic[State, Action, Event], SerializationMixin):
                 if self.store_options.on_finish:
                     self.store_options.on_finish()
                 break
-            time.sleep(0.1)
+            self.run()
 
     def _handle_finish_event(self: Store[State, Action, Event]) -> None:
         Thread(target=self._wait_for_store_to_finish).start()
@@ -439,15 +445,7 @@ class Store(Generic[State, Action, Event], SerializationMixin):
                 ReturnType,
             ],
         ) -> Callable[Args, ReturnType]:
-            def wrapper(*args: Args.args, **kwargs: Args.kwargs) -> ReturnType:
-                if self._state is None:
-                    msg = 'Store has not been initialized yet.'
-                    raise RuntimeError(msg)
-                return func(selector(self._state), *args, **kwargs)
-
-            wrapper.__name__ = f'with_state:{func.__name__}'
-
-            return wrapper
+            return WithState(store=self, selector=selector, func=cast('Callable', func))
 
         return with_state_decorator
 
